@@ -26,6 +26,9 @@ Pixie - The magic data pixie
   # Delete it
   $pixie->delete( $cookie ) || warn "eek!";
 
+  # some stores need deploying before you can use them:
+  $pixie = Pixie->deploy( 'dbi:mysql:dbname=px_test' );
+
 =head1 DESCRIPTION
 
 Pixie is yet another object persistence tool. The basic goal of Pixie
@@ -73,12 +76,13 @@ package Pixie;
 
 use strict;
 use warnings::register;
-use Carp;
+
+use Carp qw( carp confess );
 
 use Data::UUID;
+use Scalar::Util qw( blessed reftype isweak );
 use Pixie::Proxy;
 use Data::Dumper;
-use Scalar::Util qw/ blessed reftype isweak /;
 
 use Pixie::Store;
 use Pixie::ObjectInfo;
@@ -89,27 +93,23 @@ use Pixie::Complicity;
 
 use Pixie::LockStrat::Null;
 
-our $VERSION="2.08_01";
+use base qw( Pixie::Object );
+
+our $VERSION = "2.08_02";
 our $the_current_pixie;
 our $the_current_oid;
 our $the_current_lock_strategy;
 our $the_current_object_graph;
 
-use base 'Pixie::Object';
-
 #use overload
 #  '""' => 'as_string';
 
-
-sub as_string {
-  my $self = shift;
-  my $str = ref($self) . ": " . $self->_oid . "\n";
-  $str .= "   " . $self->store->as_string . "\n" if $self->store;
-}
-
 #BEGIN { $Data::Dumper::Useperl = 1 }
 
-## CLASS METHODS
+#------------------------------------------------------------------------------
+# Class methods
+#------------------------------------------------------------------------------
+
 sub get_the_current_pixie {
   my $class = shift;
   return $the_current_pixie;
@@ -128,10 +128,19 @@ sub get_the_current_object_graph {
   return $the_current_object_graph;
 }
 
+## TODO: return new object
+sub deploy {
+  my $class = shift;
+  Pixie::Store->deploy( @_ );
+  return $class;
+}
+
+#------------------------------------------------------------------------------
+# Instance methods
+#------------------------------------------------------------------------------
 
 sub init {
   my $self = shift;
-
   $self->connect('memory');
   $self->{_objectmanager} = Pixie::LiveObjectManager->new->set_pixie($self);
   return $self;
@@ -139,7 +148,7 @@ sub init {
 
 sub connect {
   my $self = shift;
-  $self = $self->new unless ref $self;
+  $self = $self->new unless blessed( $self );
   $self->store( Pixie::Store->connect(@_) );
 }
 
@@ -148,6 +157,18 @@ sub clear_storage {
   $self->store->clear;
 }
 
+#------------------------------------------------------------------------------
+# accessor-kinda methods
+
+sub _oid {
+  my $self = shift;
+  $self->{_oid} ||= do {
+    require Data::UUID;
+    Data::UUID->new()->create_str();
+  }
+}
+
+## TODO: rename to _store or the_store
 sub store {
   my $self = shift;
   if (@_) {
@@ -159,8 +180,9 @@ sub store {
 }
 
 sub clear_store {
-    my $self = shift;
+    my $self       = shift;
     $self->{store} = undef;
+    return $self;
 }
 
 sub lock_strategy {
@@ -174,8 +196,9 @@ sub lock_strategy {
   }
 }
 
+## basically an accessor for live obj manager...
 sub lock_strategy_for {
-  my $self = shift;
+  my $self       = shift;
   my $obj_or_oid = shift;
 
   if (@_) {
@@ -187,49 +210,100 @@ sub lock_strategy_for {
   }
 }
 
+#------------------------------------------------------------------------------
+# storage methods
+
+## TODO: is this actually being used?
 sub store_individual {
   my $self = shift;
   my $real = shift;
 
-  die "Trying to store something unstorable" if
-    eval { $real->isa('Pixie::ObjectInfo') };
+  confess( "Can't store a Pixie::ObjectInfo" )
+    if eval { $real->isa('Pixie::ObjectInfo') };
+
   my $oid = $real->PIXIE::oid;
   if (defined $oid) {
     $self->store_individual_at($real, $oid);
   }
   else {
+    # TODO: when are we *not* gonna have an oid?  Should die really.
     return $real;
   }
 }
 
-
+## TODO: fix order of params: everywhere else uses oid => obj.
 sub store_individual_at {
   my $self = shift;
   my($obj, $oid, $strategy) = @_;
   $strategy ||= $self->lock_strategy;
+  # TODO: is %Pixie::Stored actually used?
   if ($Pixie::Stored{$oid}) {
     return $Pixie::Stored{$oid};
   }
   else {
     return Pixie::Proxy->
-      px_make_proxy($self->store->store_at($oid, $obj,$strategy));
+      px_make_proxy( $self->store->store_at($oid, $obj, $strategy) );
   }
 }
 
-sub _oid {
+#------------------------------------------------------------------------------
+# Insert methods
+
+sub insert {
+  my $self  = shift;
+  my $graph = Pixie::ObjectGraph->new;
+
+  my $ret = eval {
+    local $the_current_object_graph = $graph;
+    $self->_insert(@_)
+  };
+  $self->bail_out($@) if $@;
+
+  $self->_insert($self->object_graph->add_graph($graph))
+    unless $_[0]->isa('Pixie::ObjectGraph');
+  $self->add_to_rootset(@_);
+
+  return $ret;
+}
+
+##
+# How Object Freezing Works
+#
+# To insert a tree of objects without disturbing the original objects
+# themselves we need to take a deep copy of the tree and extract each object
+# to be stored individualy.
+#
+# We use Data::Dumper to do this, by dumping to a string, and eval'ing it
+# straight away to get the new object tree.  We use Dumper's call-back hooks
+# Freezer & Toaster, which are used for each object in the tree.
+#
+# When an object is frozen it's wrapped in an Object Holder so we can preserve
+# its oid.
+#
+# When an object is thawed we take it out of the Holder and store it.
+##
+
+sub _insert {
   my $self = shift;
-  $self->{_oid} ||= do {
-    require Data::UUID;
-    Data::UUID->new()->create_str();
-  }
+  my $this = shift;
+
+  local %Pixie::Stored; # TODO: is %Pixie::Stored actually used?
+  local $Data::Dumper::Freezer = '_px_insertion_freeze';
+  local $Data::Dumper::Toaster = '_px_insertion_thaw';
+
+  local %PIXIE::freeze_cache;
+  my $proxy = $self->do_dump_and_eval($this, 1);
+
+  return defined($proxy) ? $proxy->_oid : undef;
 }
 
+## TODO: rename deep_copy_using_data_dumper()
 sub do_dump_and_eval {
   my $self = shift;
   my($thing, $do_lock) = @_;
 
   local $Data::Dumper::Deepcopy = 1;
-  local $the_current_pixie = $self;
+  local $the_current_pixie      = $self;
 
   my $data_string;
   {
@@ -248,48 +322,43 @@ sub do_dump_and_eval {
   eval $data_string;
   die $@ if $@;
   $self->unlock_store if $do_lock;
+
   return $VAR1;
 }
 
-sub _insert{
-  my $self = shift;
-  my $this = shift;
-
-  local %Pixie::Stored;
-  local $Data::Dumper::Freezer = '_px_insertion_freeze';
-  local $Data::Dumper::Toaster = '_px_insertion_thaw';
-
-  local %PIXIE::freeze_cache;
-  my $proxy = $self->do_dump_and_eval($this, 1);
-
-  return defined($proxy) ? $proxy->_oid : undef;
-}
-
+## TODO: split up into smaller methods with better names
 sub insertion_freeze {
-  my $self = shift;
+  my $self  = shift;
   my $thing = shift;
+
   $self->ensure_storability($thing);
   my $oid = $thing->PIXIE::oid;
+
   return $PIXIE::freeze_cache{$oid} if defined $PIXIE::freeze_cache{$oid};
+
   $self->cache_insert($thing);
   $thing = $thing->px_freeze;
-  my $obj_holder = bless {oid     => $oid,
-				      class   => ref($thing),
-				      content => $thing->px_as_rawstruct },
-					'Pixie::ObjHolder';
+
+  my $obj_holder = bless( {oid     => $oid,
+			   class   => blessed( $thing ),
+			   content => $thing->px_as_rawstruct },
+			  'Pixie::ObjHolder' );
   $PIXIE::freeze_cache{$oid} = $obj_holder;
+
+  return $obj_holder;
 }
 
-
+## TODO: split up into smaller methods with better names
 sub insertion_thaw {
-  my $self = shift;
+  my $self       = shift;
   my $obj_holder = shift;
   die "Object is not a Pixie::ObjHolder" unless $obj_holder->isa('Pixie::ObjHolder');
 
-  my $thing = bless $obj_holder->{content}, $obj_holder->{class};
+  my $thing     = bless $obj_holder->{content}, $obj_holder->{class};
   my $thing_oid = $obj_holder->{oid};
 
   $self->{_objectmanager}->bind_object_to_oid($thing, $thing_oid);
+  # TODO: this has already been ensured in insertion_freeze:
   $self->ensure_storability($thing);
   my $retval = $self->store_individual_at($thing, $obj_holder->{oid});
 
@@ -299,25 +368,33 @@ sub insertion_thaw {
       $self->proxied_content($obj_holder);
   }
   bless $thing, 'Class::Whitehole';
+
   return $retval;
 }
 
+sub proxied_content {
+  my $self       = shift;
+  my $obj_holder = shift;
 
-sub insert {
-  my $self = shift;
-  my $graph = Pixie::ObjectGraph->new;
+  local %Pixie::neighbours;
 
-  my $ret = eval {
-    local $the_current_object_graph = $graph;
-    $self->_insert(@_)
-  };
-  $self->bail_out($@) if $@;
-  $self->_insert($self->object_graph->add_graph($graph))
-    unless $_[0]->isa('Pixie::ObjectGraph');
-  $self->add_to_rootset(@_);
-  return $ret;
+  # Turn off deepcopy or things get *very* slow.
+  local $Data::Dumper::Deepcopy = 0;
+  local $Data::Dumper::Freezer  = 'Pixie::proxy_finder';
+  local $Data::Dumper::Toaster  = undef;
+  Data::Dumper::DumperX($obj_holder);
+  return keys %Pixie::neighbours;
 }
 
+sub proxy_finder {
+  my $obj = shift;
+  $Pixie::neighbours{$obj->_oid} = 1 if blessed( $obj )->isa( 'Pixie::Proxy' );
+  return $obj;
+}
+
+#------------------------------------------------------------------------------
+# Get methods
+
 sub get {
   my $self = shift;
   my($oid) = @_;
@@ -327,10 +404,12 @@ sub get {
 sub get_with_strategy {
   my $self = shift;
   my($oid, $strategy) = @_;
+
   $strategy ||= do {
     carp "Called with blank strategy";
     $self->lock_strategy;
   };
+
   local $the_current_lock_strategy = $strategy;
 
   $self->lock_store;
@@ -340,36 +419,10 @@ sub get_with_strategy {
   $strategy->post_get($oid, $self);
   $self->bail_out($err) if $err;
   $self->unlock_store;
+
   return $res;
 }
 
-sub bail_out {
-  my $self = shift;
-  $self->rollback_store;
-  $self->unlock_store;
-  die @_;
-}
-
-sub delete {
-  my $self = shift;
-  my $obj_or_oid  = shift;
-
-  my $oid = ref($obj_or_oid) ? $obj_or_oid->PIXIE::oid : $obj_or_oid;
-  $self->cache_delete($oid);
-  $self->store->remove_from_store($oid);
-}
-
-sub forget_about {
-  my $self = shift;
-  return unless ref($self);
-  my $obj = shift;
-  $obj->PIXIE::set_info(undef);
-}
-
-sub lock_store     { $_[0]->store->lock; }
-sub unlock_store   { $_[0]->store->unlock; }
-sub rollback_store { $_[0]->store->rollback; }
-
 sub _get {
   my $self = shift;
   my $oid  = shift;
@@ -392,26 +445,28 @@ sub _get {
 }
 
 sub extraction_freeze {
-  my $self = shift;
+  my $self  = shift;
   my $thing = shift;
   return $thing;
 }
 
 sub extraction_thaw {
-  my $self = shift;
+  my $self  = shift;
   my $thing = shift;
-
   my $oid   = Pixie->get_the_current_oid;
+
   $thing = $thing->px_thaw;
 
+  # this usually calls 'make_new_object':
   my $real_obj = $thing->px_do_final_restoration;
 
-  bless $thing, 'Class::Whitehole' unless
-    $thing->PIXIE::address == $real_obj->PIXIE::address;
+  bless( $thing, 'Class::Whitehole' )
+    unless $thing->PIXIE::address == $real_obj->PIXIE::address;
 
   $self->{_objectmanager}->bind_object_to_oid($real_obj => $oid);
   $real_obj->PIXIE::oid eq $oid or die "Bad OID stuff";
   $self->cache_insert($real_obj);
+
   return $real_obj;
 }
 
@@ -442,6 +497,28 @@ sub make_new_object {
   return $real;
 }
 
+sub bail_out {
+  my $self = shift;
+  $self->rollback_store;
+  $self->unlock_store;
+  die @_;
+}
+
+sub delete {
+  my $self       = shift;
+  my $obj_or_oid = shift;
+  my $oid        = blessed( $obj_or_oid ) ? $obj_or_oid->PIXIE::oid : $obj_or_oid;
+  $self->cache_delete($oid);
+  $self->store->remove_from_store($oid);
+}
+
+sub forget_about {
+  my $self = shift;
+  return unless blessed( $self );
+  my $obj = shift;
+  $obj->PIXIE::set_info(undef);
+}
+
 sub manages_object {
   my $self = shift;
   my($obj) = @_;
@@ -449,6 +526,8 @@ sub manages_object {
   $self->_oid eq $obj->PIXIE::get_info->pixie_id;
 }
 
+#------------------------------------------------------------------------------
+# Caching related methods
 
 sub cache_insert {
   my $self = shift;
@@ -476,8 +555,10 @@ sub get_cached_keys {
   $self->{_objectmanager}->cache_keys;
 }
 
+#------------------------------------------------------------------------------
 # The naming section
 
+## TODO: just use Pixie::Name by default.
 sub bind_name {
   my $self = shift;
   my($name, @objects) = @_;
@@ -487,7 +568,7 @@ sub bind_name {
 }
 
 sub unbind_name {
-  my $self = shift;
+  my $self  = shift;
   my($name) = @_;
 
   require Pixie::Name;
@@ -502,8 +583,8 @@ sub get_object_named {
 }
 
 
-
-# GC related stuff
+#------------------------------------------------------------------------------
+# Garbage Collection & related
 
 sub rootset {
   my $self = shift;
@@ -516,33 +597,12 @@ sub add_to_rootset {
   return $self;
 }
 
-sub proxy_finder {
-  my $obj = shift;
-  $Pixie::neighbours{$obj->_oid} = 1 if ref($obj)->isa('Pixie::Proxy');
-  return $obj;
-}
-
-sub proxied_content {
-  my $self = shift;
-  my $obj_holder = shift;
-
-  local %Pixie::neighbours;
-
-  # Turn off deepcopy or things get *very* slow.
-  local $Data::Dumper::Deepcopy = 0;
-  local $Data::Dumper::Freezer = 'Pixie::proxy_finder';
-  local $Data::Dumper::Toaster = undef;
-  Data::Dumper::DumperX($obj_holder);
-  return keys %Pixie::neighbours;
-}
-
 sub neighbours {
   my $self = shift;
-  my $oid = shift;
+  my $oid  = shift;
   $self->object_graph->neighbours($oid);
 }
 
-
 sub run_GC {
   my $self = shift;
   $self->store->lock_for_GC;
@@ -555,9 +615,9 @@ sub run_GC {
 }
 
 sub live_set {
-  my $self = shift;
+  my $self  = shift;
   my $graph = $self->object_graph;
-  my %seen = ();
+  my %seen  = ();
   my @nodes_to_process = $self->rootset;
 
   while (@nodes_to_process) {
@@ -566,26 +626,33 @@ sub live_set {
     $seen{$node} = 1;
     push @nodes_to_process, $graph->neighbours($node);
   }
+
   return keys %seen;
 }
 
 sub object_graph {
   my $self = shift;
-  $self->store->object_graph_for($self);
+  $self->store->object_graph_for( $self );
 }
 
 sub working_set {
   my $self = shift;
-  $self->store->working_set_for($self);
+  $self->store->working_set_for( $self );
 }
 
 sub ensure_storability {
   my $self = shift;
-  my $obj = shift;
-
-  $obj->px_is_storable or die "Pixie cannot store a ", ref($obj);
+  my $obj  = shift;
+  $obj->px_is_storable or die "Pixie cannot store a ", blessed( $obj );
 }
 
+#------------------------------------------------------------------------------
+# Locking methods
+
+sub lock_store     { $_[0]->store->lock; }
+sub unlock_store   { $_[0]->store->unlock; }
+sub rollback_store { $_[0]->store->rollback; }
+
 sub lock_object {
   my $self = shift;
   $self->{_objectmanager}->lock_object(@_);
@@ -596,15 +663,12 @@ sub unlock_object {
   $self->{_objectmanager}->unlock_object(@_);
 }
 
-sub DESTROY {
-  my $self = shift;
-  $self->store->release_all_locks if defined $self->store;
-  delete $self->{_objectmanager};
-}
+#------------------------------------------------------------------------------
+# Pixie::Complicity methods
 
 sub px_freeze {
   my $self = shift;
-  return bless {}, ref($self);
+  return bless {}, blessed( $self );
 }
 
 sub _px_extraction_thaw {
@@ -612,6 +676,20 @@ sub _px_extraction_thaw {
   $self->get_the_current_pixie;
 }
 
+#------------------------------------------------------------------------------
+# Miscellaneous methods
+
+sub DESTROY {
+  my $self = shift;
+  $self->store->release_all_locks if defined $self->store;
+  delete $self->{_objectmanager};
+}
+
+sub as_string {
+  my $self = shift;
+  my $str  = blessed( $self ) . ": " . $self->_oid . "\n";
+  $str    .= "   " . $self->store->as_string . "\n" if $self->store;
+}
 
 1;
 
@@ -625,15 +703,22 @@ At the time of writing the following stores were available:
 
 =item Memory
 
+Simple memory store, good for testing.  See L<Pixie::Store::Memory>.
+
   $pixie->connect( 'memory' );
 
 =item Berkeley DB
 
+A Berkeley DB store, also good for testing (especially if you want to store
+values across tests).  See L<Pixie::Store::BerkeleyDB>.
+
   $pixie->connect( "bdb:$path_to_dbfile" );
 
 =item DBI
 
-See L<DBI> and individual DBD drivers for details on dbi's DSN specs.
+DBI-based stores are good for production.  See L<Pixie::Store::DBI> and
+subclasses, and L<DBI> for details on DSN specs to use.
+
 In general:
 
   $pixie->connect( $dbi_spec, %args );
@@ -647,7 +732,7 @@ For example:
 =back
 
 See L<Pixie::Store> and its sub-classes for more details on the available types
-of stores and the DSN specs to use.
+of stores and the DSN's to use.
 
 =head1 CONSTRUCTOR
 
@@ -664,10 +749,16 @@ really do anything.
 
 =over 4
 
+=item $px->deploy( $dsn [, @args ] )
+
+Deploy a Pixie store to the specified $dsn.  This is not required for all types
+of store (see L<Pixie::Store> and subclasses), but deploying to those stores
+won't hurt so if you want to make your code generic then go for it.
+
 =item $px->connect( $dsn [, @args ] )
 
-Connect the pixie to a store specified by $dsn (data source name).  Note that
-you may need to initialize the store before connecting to it.
+Connect the pixie to a store specified by $dsn.  Note that you may need to
+L<deploy()> the store before connecting to it.
 
 =item $cookie = $px->insert( $object );
 
@@ -678,7 +769,7 @@ $object in the future.
 
 Get the object associated with $cookie from the pixie's store.
 
-=item $px->delete( $obj_or_cookie )
+=item $px->delete( $obj || $cookie )
 
 Delete an object from the pixie's store given a $cookie, or the $object
 itself.
@@ -689,7 +780,7 @@ Gives a $name to the $object you've specified, so that you can retrieve it
 in the future using L<get_object_named()>.
 
 Returns the cookie of the C<Pixie::Name> associated with the $object, though
-B<this usage is deprecated and will likely in the next release>.
+B<this usage is deprecated and will likely be removed in the next release>.
 
 =item $obj = $px->get_object_named( $name )
 
@@ -759,7 +850,7 @@ Steve Purkis <spurkis@cpan.org> is helping to maintain the module.
 
 =head1 COPYRIGHT
 
-Copyright 2002 Fotango Ltd
+Copyright (c) 2002-2004 Fotango Ltd.
 
 This software is released under the same license as Perl itself.
 
