@@ -1,55 +1,68 @@
+=head1 NAME
+
+Pixie::Store::Memory -- a memory store for Pixie.
+
+=head1 SYNOPSIS
+
+  use Pixie;
+  my $px = Pixie->new->connect( 'memory' );
+
+=head1 DESCRIPTION
+
+A memory store for Pixie.
+
+=cut
+
 package Pixie::Store::Memory;
 
-our $VERSION="2.06";
+use Carp qw( confess );
+use Storable qw( nfreeze thaw );
 
-use Storable qw/nfreeze thaw/;
+use base qw( Pixie::Store );
 
-use base qw/Pixie::Store/;
-
-sub new {
-  my $proto = shift;
-  my $self = bless {}, $proto;
-  $self->init;
-  return $self;
-}
+our $VERSION = "2.08_02";
 
 sub init {
   my $self = shift;
-  $self->{store} = {};
+  $self->{store}   = {};
   $self->{rootset} = {};
   return $self;
 }
 
+sub deploy {
+  my $self = shift;
+  # do nothing
+  return $self;
+}
 
 sub connect {
   my $self = shift;
-  $self = ref($self) ? $self : $self->new;
+  $self    = ref($self) ? $self : $self->new;
 }
 
 sub remove_from_rootset {
   my $self = shift;
-  my $oid = shift;
+  my $oid  = shift;
   delete $self->{rootset}{$oid};
   return $self;
 }
 
-
 sub _add_to_rootset {
-  my $self = shift;
+  my $self  = shift;
   my $thing = shift;
   $self->{rootset}{$thing->PIXIE::oid} = 1;
   return $self;
 }
 
+## TODO: use wantarray
 sub rootset {
   my $self = shift;
   keys %{$self->{rootset}};
 }
 
-
 sub working_set_for {
   my $self = shift;
-  my @ret = keys %{$self->{store}};
+  my @ret  = keys %{$self->{store}};
   return wantarray ? @ret : \@ret;
 }
 
@@ -63,6 +76,7 @@ sub store_at {
   my $self = shift;
   my($oid, $obj) = @_;
 
+  # TODO: why not throw an error if no $oid?
   if ($oid) {
     $self->{store}{$oid} = nfreeze($obj);
     return($oid, $obj);
@@ -79,14 +93,13 @@ sub get_object_at {
   return thaw $self->{store}{$oid};
 }
 
-sub lock { }
-sub unlock { }
-sub rollback { }
+sub lock { $_[0] }
+sub unlock { $_[0] }
+sub rollback { $_[0] }
 
 sub clear {
   my $self = shift;
-
-  %{$self->{store}} = ();
+  %{$self->{store}}   = ();
   %{$self->{rootset}} = ();
   return $self;
 }
@@ -104,3 +117,24 @@ sub delete_object_at {
 }
 
 1;
+
+__END__
+
+=head1 SEE ALSO
+
+L<Pixie>, L<Pixie::Store>
+
+=head1 AUTHORS
+
+James Duncan <james@fotango.com>, Piers Cawley <pdcawley@bofh.org.uk>
+and Leon Brocard <acme@astray.com>.
+
+Docs by Steve Purkis <spurkis@cpan.org>.
+
+=head1 COPYRIGHT
+
+Copyright (c) 2002-2004 Fotango Ltd
+
+This software is released under the same license as Perl itself.
+
+=cut
